//+------------------------------------------------------------------+
//|                                               LaguerreSignal.mqh |
//|                             Copyright  2011,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, Nikolay Kositsin"
#property link      "farria@mail.redcom.ru"
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
#property tester_indicator "ColorLaguerre.ex5"
#include <Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+ 
//|                                                |
//+------------------------------------------------------------------+ 
#define OPEN_LONG     80  //          
#define OPEN_SHORT    80  //          
#define CLOSE_LONG    40  //          
#define CLOSE_SHORT   40  //          
#define REVERSE_LONG  100 //         
#define REVERSE_SHORT 100 //         
#define NO_SIGNAL      0  //         
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=,    ColorLaguerre            |
//| Type=SignalAdvanced                                              |
//| Name=ColorLaguerre                                               |
//| Class=CLaguerreSignal                                            |
//| Page=                                                            |
//| Parameter=BuyPosOpen,bool,true,           |
//| Parameter=SellPosOpen,bool,true,          |
//| Parameter=BuyPosClose,bool,true,      |
//| Parameter=SellPosClose,bool,true,     |
//| Parameter=Ind_Timeframe,ENUM_TIMEFRAMES,PERIOD_H4,      |
//| Parameter=gamma,double,0.7,                 |
//| Parameter=HighLevel,uint,85,                  |
//| Parameter=MiddleLevel,uint,50,                     |
//| Parameter=LowLevel,uint,15,                    |
//| Parameter=SignalBar,uint,1,              |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//|  CLaguerreSignal.                                           |
//| :          |
//| ColorLaguerre http://www.mql5.com/ru/code/432/.                  |
//|     CExpertSignal.                    |
//+------------------------------------------------------------------+
class CLaguerreSignal : public CExpertSignal
  {
protected:
   CiCustom          m_indicator;        //      Laguerre

   //---  
   bool              m_BuyPosOpen;       //     
   bool              m_SellPosOpen;      //     
   bool              m_BuyPosClose;      //     
   bool              m_SellPosClose;     //     
   ENUM_TIMEFRAMES   m_Ind_Timeframe;    //   Laguerre
   double            m_gamma;            //  
   uint              m_HighLevel;        //  
   uint              m_MiddleLevel;      //  
   uint              m_LowLevel;         //  
   uint              m_SignalBar;        //       

public:
                     CLaguerreSignal();

   //---    
   void              BuyPosOpen(bool value)                  { m_BuyPosOpen=value;       }
   void              SellPosOpen(bool value)                 { m_SellPosOpen=value;      }
   void              BuyPosClose(bool value)                 { m_BuyPosClose=value;      }
   void              SellPosClose(bool value)                { m_SellPosClose=value;     }
   void              Ind_Timeframe(ENUM_TIMEFRAMES value)    { m_Ind_Timeframe=value;    }
   void              gamma(double value)                     { m_gamma=value;            }
   void              HighLevel(uint value)                   { m_HighLevel=value;        }
   void              MiddleLevel(uint value)                 { m_MiddleLevel=value;      }
   void              LowLevel(uint value)                    { m_LowLevel=value;         }
   void              SignalBar(uint value)                   { m_SignalBar=value;        }

   //---    
   virtual bool      ValidationSettings();
   //---    
   virtual bool      InitIndicators(CIndicators *indicators); //  
   //---      
   virtual int       LongCondition();
   virtual int       ShortCondition();

   bool              InitColorLaguerre(CIndicators *indicators); // ,   Laguerre
  };
//+------------------------------------------------------------------+
//|  CLaguerreSignal.                                     |
//| INPUT:  .                                                     |
//| OUTPUT: .                                                     |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
void CLaguerreSignal::CLaguerreSignal()
  {
//---   -
   m_BuyPosOpen=true;
   m_SellPosOpen=true;
   m_BuyPosClose=true;
   m_SellPosClose=true;
   m_Ind_Timeframe=PERIOD_H4;
   m_gamma=0.7;
   m_HighLevel=85;
   m_MiddleLevel=50;
   m_LowLevel=15;
   m_SignalBar=1;
   m_used_series=USE_SERIES_OPEN+USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE;
  }
//+------------------------------------------------------------------+
//|   .                                   |
//| INPUT:  .                                                     |
//| OUTPUT: true-  ,  false.             |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CLaguerreSignal::ValidationSettings()
  {
//---  
   if(m_HighLevel<=0)
     {
      printf(__FUNCTION__+":      ");
      return(false);
     }

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|    .                           |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CLaguerreSignal::InitIndicators(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---  
   if(!InitColorLaguerre(indicators)) return(false);

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|   Laguerre.                               |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CLaguerreSignal::InitColorLaguerre(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---    
   if(!indicators.Add(GetPointer(m_indicator)))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---   
   MqlParam parameters[5];

   parameters[0].type=TYPE_STRING;
   parameters[0].string_value="ColorLaguerre.ex5";

   parameters[1].type=TYPE_DOUBLE;
   parameters[1].double_value=m_gamma;

   parameters[2].type=TYPE_UINT;
   parameters[2].integer_value=m_HighLevel;

   parameters[3].type=TYPE_UINT;
   parameters[3].integer_value=m_MiddleLevel;
   
   parameters[4].type=TYPE_UINT;
   parameters[4].integer_value=m_LowLevel;

//---     
   if(!m_indicator.Create(m_symbol.Name(),m_Ind_Timeframe,IND_CUSTOM,5,parameters))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }
     
//---  
   if(!m_indicator.NumBuffers(2)) return(false);
   
//---     Laguerre
   return(true);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CLaguerreSignal::LongCondition()
  {
//---       1  Laguerre
   double Signal0=m_indicator.GetData(1,m_SignalBar);
   double Signal1=m_indicator.GetData(1,m_SignalBar+1);

//---    
   if(Signal0==1)
     {
      if(Signal1==2)
        {

         if(m_BuyPosOpen)
           {
            if(m_SellPosClose) return(REVERSE_SHORT);
            else return(OPEN_LONG);
           }

         if(m_SellPosClose) return(CLOSE_SHORT);
        }
     }
//---   
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CLaguerreSignal::ShortCondition()
  {
//---       1  Laguerre
   double Signal0=m_indicator.GetData(1,m_SignalBar);
   double Signal1=m_indicator.GetData(1,m_SignalBar+1);

//---    
   if(Signal0==2)
     {
      if(Signal1==1)
        {

         if(m_SellPosOpen)
           {
            if(m_BuyPosClose) return(REVERSE_LONG);
            else return(OPEN_SHORT);
           }

         if(m_BuyPosClose) return(CLOSE_LONG);
        }
     }
//---      
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
